import React from 'react';
import type { MiniMapNodeProps } from './types';
declare const _default: React.MemoExoticComponent<{
    ({ id, x, y, width, height, style, color, strokeColor, strokeWidth, className, borderRadius, shapeRendering, onClick, selected, }: MiniMapNodeProps): JSX.Element;
    displayName: string;
}>;
export default _default;
//# sourceMappingURL=MiniMapNode.d.ts.map