import React from 'react';
import type { MiniMapNodes } from './types';
declare function MiniMapNodes({ nodeStrokeColor, nodeColor, nodeClassName, nodeBorderRadius, nodeStrokeWidth, nodeComponent: NodeComponent, onClick, }: MiniMapNodes): JSX.Element;
declare const _default: React.MemoExoticComponent<typeof MiniMapNodes>;
export default _default;
//# sourceMappingURL=MiniMapNodes.d.ts.map